'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = objectOf;

function objectOf(model) {
  return {
    getMatcher: function getMatcher() {
      var matcher = model.getMatcher();

      return function (a, b) {
        var aType = typeof a;
        var bType = typeof b;

        if (aType !== bType) {
          return false;
        }

        if (aType !== 'object' || a === null || b === null) {
          return a === b;
        }

        var aKeys = Object.keys(a);
        var bKeys = Object.keys(b);

        if (aKeys.length !== bKeys.length) {
          return false;
        }

        for (var i = 0, l = aKeys.length; i < l; ++i) {
          // suspect that aKeys and bKeys are the same
          // or at least they values will be different
          var key = aKeys[i];
          if (!matcher(a[key], b[key])) {
            return false;
          }
        }

        return true;
      };
    },

    getMerger: function getMerger() {
      var merger = model.getMerger();

      return function (a, b, connect) {
        if (typeof b === 'function') {
          b = b(connect);
        }

        var aType = typeof a;
        var bType = typeof b;

        if (bType !== 'object') {
          return a;
        }

        if (b === null) {
          return b;
        }
        //so b is an object and not null
        if (aType !== 'object' || a === null) {
          a = {};
        }
        //so a and b both are real objects
        var bKeys = Object.keys(b);
        for (var i = 0, l = bKeys.length; i < l; i++) {
          var key = bKeys[i];
          a[key] = merger(a[key], b[key]);
        }
        return a;
      };
    }
  };
}

module.exports = exports['default'];